package Src;
import java.io.* ;

/**
 *  Classe permettant de lire un fichier d'ar???tes.
 *
 *  Chaque ligne du fichier est de la forme :
 *  2 --- 3
 *  sommet1 --- sommet2
 */

public class LecteurAretes {

    private BufferedReader in ;
    private boolean open ;
    private String Filename ;
    private int linenb ;

    /** Contient le premier sommet de l'ar???te. */
    public int sommet1 ;

    /** Contient le second sommet de l'ar???te. */
    public int sommet2 ;


    /**
     * @param  Filename  Nom du fichier contenant les ar???tes.
     */
    public LecteurAretes (String Filename) throws FileNotFoundException{
	this.Filename = Filename ;
	this.in = new BufferedReader(new FileReader(Filename)) ;
	this.open = true ;
	this.linenb = 0 ;
    }



    /**
     * Lit l'ar???te suivante dans le fichier.
     * Les deux sommets de l'ar???te sont plac???s dans les attributs de l'objet.
     * Cette m???thode peut ???tre invoqu???e sans risque d'erreur m???me s'il ne reste plus d'ar???tes ??? lire.
     *
     * @return      true si une ar???te a bien ???t??? lue.
     */

    public boolean areteSuivante () {
	if (this.open) {
	    try {
		String line = in.readLine () ;
		if (line == null) return false ;

		line = line.trim () ;
		linenb = linenb + 1 ;

		if ((line.length () == 0) || (this.is_comment (line)) ) { return this.areteSuivante () ; }

		String[] splits = line.split ("---", 2) ;
		
		sommet1 = this.get_int ("du premier sommet", splits[0].trim ()) ;
		sommet2 = this.get_int ("du second sommet", splits[1].trim ()) ;
		
		return true ;
	    }
	    catch (IOException e) {
		open = false ;
		try { in.close () ; } catch (IOException ee) {}
		return false ;
	    }
	}
	else return false ;
    }

    private int get_int (String champ, String v) {
	try {return ((new Integer (v)).intValue ()) ;}
	catch (NumberFormatException e) {
	    System.err.println("Fichier " + Filename + ", ligne " + linenb + " : erreur de lecture " + champ) ;
	    return 0 ;
	}
    }

    private boolean is_comment (String line) {
	return line.startsWith("//") ;
    }

    //Pour charger la liste des Aretes
    public static void lireAretes(String nomFic, Graphe graph) throws FileNotFoundException {
	LecteurAretes lect = new LecteurAretes (nomFic) ;
	Sommet som1;
	Sommet som2;
	double pente;
	double distance;
	
	while (lect.areteSuivante ()) {
		//R??cup??ration des sommets concern??s
		som2 = graph.getListeSommets().get(lect.sommet2);
		som1 = graph.getListeSommets().get(lect.sommet1);
		//calcul des diff??rences entre les coordonn??es
		double difAltitude = som2.getAltitude()-som1.getAltitude();
		//calcul de la distance entre les villes (en prenant en compte l'altitude???)
		distance = Algo.degreesToKm(som1.getLatitude(),som1.getLongitude(),som2.getLatitude(),som2.getLongitude());
		//Calcul de la pente enter les deux villes
		pente = Math.atan(difAltitude/distance);

		//Calcul de la vitesse
		//vitesse = 75 ??? Math.min(30, pente * 5);
		
		//Rajout des deux succ??sseurs dans le graphe (som1 vers som2 et som2 vers som1)
		graph.ajouterSuccesseur(new Successeur(som2.getIdSom(), new ValuationArc(distance, pente)), som1);
		graph.ajouterSuccesseur(new Successeur(som1.getIdSom(), new ValuationArc(distance, -pente)), som2);
		//System.out.println ("Ar???te " + lect.sommet1 + " <-> " + lect.sommet2) ;
	}
    }
    

    
    //Pour le nb de Successeurs par sommet
    public static int[] lireNbSuccMax(String nomFic, int nbSommets) throws FileNotFoundException {
    	LecteurAretes lect = new LecteurAretes (nomFic) ;
    	int[] tab = new int[nbSommets];
    	for(int i =0; i<nbSommets ; i++){
    		tab[i]=0;
    	}
    	while (lect.areteSuivante ()) {
    		//calcul des diff??rences entre les coordonn??es
    		tab[lect.sommet1]++;
    		tab[lect.sommet2]++;
    	}
    	return tab;
    }
    

}